#include "StdAfx.h"
#include "Graphics.h"

#include <sstream>
#include <pugixml.hpp>
#ifdef _DEBUG
#pragma comment(lib, "pugixml_d.lib")
#else
#pragma comment(lib, "pugixml.lib")
#endif

#include "VertexTypes.h"

using namespace std;

Graphics::Graphics()
{
	m_pD3D = NULL;
	m_pd3dDevice = NULL;
	ZeroMemory(&m_d3dpp, sizeof(D3DPRESENT_PARAMETERS)); // 
}

Graphics::~Graphics( void )
{
}

bool Graphics::Init(HWND hRenderWnd, int width, int height)
{
	//    DirectX 9
	m_pD3D = Direct3DCreate9( D3D_SDK_VERSION );
	if (m_pD3D==NULL)
		return false;

	m_d3dpp.BackBufferWidth = width; //   
	m_d3dpp.BackBufferHeight = height; //   
	m_d3dpp.BackBufferCount = 1; //   
	m_d3dpp.Windowed = TRUE; //   ,     
	m_d3dpp.BackBufferFormat = D3DFMT_X8R8G8B8;// D3DFMT_UNKNOWN; //    DX
	m_d3dpp.SwapEffect = D3DSWAPEFFECT_DISCARD; //    
	m_d3dpp.EnableAutoDepthStencil = TRUE; //   Z-
	m_d3dpp.AutoDepthStencilFormat = D3DFMT_D24S8; //  Z-
	m_d3dpp.PresentationInterval = D3DPRESENT_INTERVAL_ONE;

	//     
	HRESULT hr = m_pD3D->CreateDevice( D3DADAPTER_DEFAULT,
		D3DDEVTYPE_HAL, hRenderWnd, D3DCREATE_HARDWARE_VERTEXPROCESSING,
		&m_d3dpp, &m_pd3dDevice );
	if (FAILED(hr))
		return FALSE;

	m_pd3dDevice->CreateVertexDeclaration(dwDeclMyVertex, &m_pVertexDeclaration);

	return TRUE;
}


void Graphics::Cleanup()
{
	m_pVertexDeclaration->Release();

	//  
	vecTextures::iterator itr = m_vecTextures.begin();
	while(itr!=m_vecTextures.end())
	{
		(*itr)->Release();
		itr++;
	}
	m_vecTextures.clear();
	
	//  
	vecShaders::iterator itr2 = m_vecShaders.begin();
	while(itr2!=m_vecShaders.end())
	{
		(*itr2)->Release();
		itr2++;
	}
	m_vecShaders.clear();

	//  
	vector<CMeshPtr>::iterator itr3 = m_vecMeshes.begin();
	while(itr3!=m_vecMeshes.end())
	{
		delete *itr3;
		itr3++;
	}
	m_vecMeshes.clear();

	//  IB/VB
	vector<LPDIRECT3DINDEXBUFFER9>::iterator itr4 = m_vecIB.begin();
	while(itr4!=m_vecIB.end())
	{
		(*itr4)->Release();
		itr4++;
	}
	m_vecIB.clear();

	vector<LPDIRECT3DVERTEXBUFFER9>::iterator itr5 = m_vecVB.begin();
	while(itr5!=m_vecVB.end())
	{
		(*itr5)->Release();
		itr5++;
	}
	m_vecVB.clear();

	if (m_pd3dDevice)
	{
		m_pd3dDevice->Release();
		m_pd3dDevice = NULL;
	}
	if (m_pD3D)
	{
		m_pD3D->Release();
		m_pD3D = NULL;
	}
}

bool Graphics::StartRender(D3DCOLOR bgColor)
{
	if (FAILED(m_pd3dDevice->BeginScene())) //  
		return false;

	 //  
	if (FAILED(m_pd3dDevice->Clear(0, NULL, D3DCLEAR_TARGET|
		D3DCLEAR_STENCIL| D3DCLEAR_ZBUFFER, bgColor, 1, 0)))
		return false;

	return true;
}

HRESULT Graphics::SetFVF( DWORD FVF )
{
	return m_pd3dDevice->SetFVF(FVF);
}

HRESULT Graphics::SetTexture( DWORD stage, CTexturePtr pTex )
{
	if (pTex)
		return m_pd3dDevice->SetTexture(stage, pTex->GetTexture());
	
	return m_pd3dDevice->SetTexture(stage, NULL);
}

bool Graphics::EndRender()
{
	m_pd3dDevice->EndScene(); //  

	//   
	HRESULT hr = m_pd3dDevice->Present(NULL, NULL, NULL, NULL);
	if (FAILED(hr)/*==D3DERR_DEVICELOST*/)
		Reset();
	
	if (FAILED(hr))
		return false;
	
	return true;
}

LPDIRECT3DDEVICE9 Graphics::GetDevice()
{
	return m_pd3dDevice;
}

CTexturePtr Graphics::LoadTexture(const std::wstring& szFileName )
{
	//     
	vecTextures::iterator itr = m_vecTextures.begin();
	while(itr!=m_vecTextures.end())
	{
		if ( (*itr)->GetSrc().compare(szFileName)==0 )
			return *itr; // 
		itr++;
	}
	
	//  ,   
	LPDIRECT3DTEXTURE9 pTex = NULL;
	if (FAILED(D3DXCreateTextureFromFile(m_pd3dDevice, szFileName.c_str(), &pTex)))
		return NULL;
	
	//     
	CTexturePtr pTexPtr = new CTexture();
	pTexPtr->SetSrc(szFileName);
	pTexPtr->SetTexture(pTex);

	m_vecTextures.push_back(pTexPtr);
	
	return pTexPtr; //  ( )  
}

HRESULT Graphics::SetRenderState( D3DRENDERSTATETYPE State, DWORD Value )
{
	return m_pd3dDevice->SetRenderState(State, Value);
}

HRESULT Graphics::SetSamplerState( DWORD Sampler, D3DSAMPLERSTATETYPE Type, DWORD Value )
{
	return m_pd3dDevice->SetSamplerState(Sampler, Type, Value);
}

HRESULT Graphics::DPUP( D3DPRIMITIVETYPE PrimitiveType, UINT PrimitiveCount, CONST void* pVertexStreamZeroData, UINT VertexStreamZeroStride )
{
	return m_pd3dDevice->DrawPrimitiveUP(PrimitiveType, PrimitiveCount, pVertexStreamZeroData, VertexStreamZeroStride);
}


HRESULT Graphics::DIPUP( D3DPRIMITIVETYPE PrimitiveType, UINT MinVertexIndex, UINT NumVertices, UINT PrimitiveCount, const void *pIndexData, D3DFORMAT IndexDataFormat, const void *pVertexStreamZeroData, UINT VertexStreamZeroStride )
{
	return m_pd3dDevice->DrawIndexedPrimitiveUP(PrimitiveType, MinVertexIndex, NumVertices, PrimitiveCount, pIndexData, IndexDataFormat, pVertexStreamZeroData, VertexStreamZeroStride);
}


HRESULT Graphics::SetPixelShader( CShaderPtr shader )
{
	return m_pd3dDevice->SetPixelShader(shader->GetPixelShader());
}

HRESULT Graphics::SetVertexShader( CShaderPtr shader )
{
	return m_pd3dDevice->SetVertexShader(shader->GetVertexShader());
}

CShaderPtr Graphics::LoadShader( const std::string& strFileName, const std::string& strFuncName, const std::string& profile )
{
	CShaderPtr pShader = new CShader;
	if (FAILED(pShader->Load(strFileName, strFuncName, profile)))
		return NULL;

	LPD3DXBUFFER pShaderBuff = pShader->GetCompileBuffer();
	if (pShader->IsPS())
	{
		//   
		LPDIRECT3DPIXELSHADER9 pPixelShader;
		m_pd3dDevice->CreatePixelShader(( DWORD* )pShaderBuff->GetBufferPointer(), &pPixelShader);
		pShader->SetHardwareShader(m_pd3dDevice, pPixelShader);
	}
	else
	{
		//   
		LPDIRECT3DVERTEXSHADER9 pVertexShader;
		m_pd3dDevice->CreateVertexShader(( DWORD* )pShaderBuff->GetBufferPointer(), &pVertexShader);
		pShader->SetHardwareShader(m_pd3dDevice, pVertexShader);
	}
	
	m_vecShaders.push_back(pShader);
	return pShader;
}

HRESULT Graphics::Reset()
{
	//  
	vecTextures::iterator itr = m_vecTextures.begin();
	while(itr!=m_vecTextures.end())
	{
		(*itr)->Release();
		itr++;
	}

	//     -    

	//  
	HRESULT hr = m_pd3dDevice->Reset(&m_d3dpp);
	if (FAILED(hr))
		return hr;

	//   
	itr = m_vecTextures.begin();
	while(itr!=m_vecTextures.end())
	{
		(*itr)->Reload(m_pd3dDevice);
		itr++;
	}

	return D3D_OK;
}

bool Graphics::LoadSpriteList( const std::string& strFileName )
{
	const std::string& strContents = GetFileAsString(strFileName);
	pugi::xml_document doc;
	pugi::xml_parse_result result = doc.load_buffer_inplace((void*)strContents.c_str(), strContents.length());
	if (!result)
		return false; //   

	stringstream ss;
	//  
	for (pugi::xml_node enemyNode = doc.child("enemy"); enemyNode; enemyNode = enemyNode.next_sibling("enemy"))
	{
		ss << "Enemy " << enemyNode.attribute("name").value() << "\n";
	}
	return true;
}

LPDIRECT3DINDEXBUFFER9 Graphics::CreateIndexBuffer(UINT Length, DWORD Usage, D3DFORMAT Format, D3DPOOL Pool)
{
	LPDIRECT3DINDEXBUFFER9 pIB = NULL;
	HRESULT hr = m_pd3dDevice->CreateIndexBuffer(Length, Usage, Format, Pool, &pIB, NULL);
	if (FAILED(hr))
		return NULL;
	
	m_vecIB.push_back(pIB);
	return pIB;
}

bool Graphics::Release(LPDIRECT3DINDEXBUFFER9 pIB)
{
	for(size_t i=0; i<m_vecIB.size(); i++)
		if (m_vecIB[i]==pIB)
		{
			//  VertexBuffer
			std::swap(m_vecIB[i], *m_vecIB.rbegin());
			pIB->Release();
			m_vecIB.pop_back();
			//  
			return true;
		}
	//     IndexBuffer
	return false;
}


LPDIRECT3DVERTEXBUFFER9 Graphics::CreateVertexBuffer(UINT Length, DWORD Usage, D3DPOOL Pool)
{
	LPDIRECT3DVERTEXBUFFER9 pVertexBuffer = NULL;
	HRESULT hr = m_pd3dDevice->CreateVertexBuffer(Length, Usage, 0, Pool, &pVertexBuffer, NULL);
	if (FAILED(hr))
		return NULL;
	
	m_vecVB.push_back(pVertexBuffer);
	return pVertexBuffer;
}

bool Graphics::Release(LPDIRECT3DVERTEXBUFFER9 pVB)
{
	for(size_t i=0; i<m_vecVB.size(); i++)
		if (m_vecVB[i]==pVB)
		{
			//  VertexBuffer
			std::swap(m_vecVB[i], *m_vecVB.rbegin());
			pVB->Release();
			m_vecVB.pop_back();
			//  
			return true;
		}
	//     VertexBuffer
	return false;
}

CMeshPtr Graphics::LoadMesh( const std::string& szFileName )
{
	CMeshPtr pMesh = new CMesh();
	if (!pMesh->Load(szFileName))
	{
		delete pMesh;
		return NULL;
	}
	m_vecMeshes.push_back(pMesh);
	return pMesh;
}

HRESULT Graphics::Draw(CMeshPtr pMesh )
{
	m_pd3dDevice->SetVertexDeclaration(m_pVertexDeclaration);
	m_pd3dDevice->SetStreamSource(0, pMesh->GetVB(), 0, sizeof(MyVertex));
	m_pd3dDevice->SetIndices(pMesh->GetIB());
	return pMesh->Draw(m_pd3dDevice);
}
