#include "stdafx.h"
#include "AIBase.h"
#include "Utils.h"
#include "LiveObject.h"
#include <iostream>
#include <fstream>
#include <sstream>

#ifndef _DEBUG
#pragma comment(lib, "lua.lib")
#pragma comment(lib, "luabind.lib")
#else
#pragma comment(lib, "lua-debug.lib")
#pragma comment(lib, "luabind-debug.lib")
#endif

using namespace std;
using namespace luabind;

lua_State *myLuaState = 0;
//!   ,   
std::vector<string> vecEnemyTypes;

void lua_stacktrace(lua_State* L)
{
	lua_Debug entry;
	int depth = 0; 

	while (lua_getstack(L, depth, &entry))
	{
		int status = lua_getinfo(L, "Sln", &entry);
		assert(status);

		for (int i=0; i<depth; i++)
			std::cout<<"\t";

		std::cout<<"in: ["<<std::string(entry.short_src)<<"] line: "<<entry.currentline<<"] src:"<<(entry.name ? std::string(entry.name) : "?")<<"\n";
		//		printf("%s(%d): %s\n", entry.short_src, entry.currentline, entry.name ? entry.name : "?");
		depth++;
	}
}

void Log(const std::string str)
{
	std::ofstream out("a.out", ios::app);
	out<<str<<"\n";
}

class GameObject
{
public:
	GameObject()
	{
		stringstream ss; 
		ss<<"Constructor :"<<(long)this;
		Log(ss.str());
	}
	~GameObject()
	{
		stringstream ss; 
		ss<<"Destructor :"<<(long)this;
		Log(ss.str());
	}
	bool IsAlive() { return true; }
};

bool InitScripts()
{
	try {
		myLuaState = lua_open();
		luabind::open(myLuaState);
		luaL_openlibs(myLuaState);

		module(myLuaState)
			[
				def ("Log", (void(*)(const std::string&))&Log),
				class_<GameObject>("LiveObj")
				.def(constructor<>())
				.def("IsAlive", &GameObject::IsAlive)
			];

		// Define a lua function that we can call
		if (luaL_dostring(myLuaState, GetFileAsString("ai.lua").c_str()))
		{
			const char* err = lua_tostring(myLuaState, -1);
			lua_close(myLuaState);
			return false;
		}
	}
	catch(const std::exception &TheError)
	{
		const char* a = lua_tostring(myLuaState, -1);
		std::cout << "LUA error:" << a << "\n";
		lua_pop(myLuaState, 1);
		lua_stacktrace(myLuaState);
		TheError.what();
		//			std::cout << "LUA error: "<<std::string(TheError.what()) << "\n";
	}

	std::stringstream ss;
	const luabind::object& object = luabind::globals(myLuaState)["Enemy01"];
	luabind::object o = luabind::call_function<luabind::object>(myLuaState, "MakeNewObjectFromLuaAndCPP", object, new GameObject())[ adopt(_2) ];

	Log("<GC>");
	lua_gc(myLuaState, LUA_GCCOLLECT, 0);
	
	Log("<Call>");
	o = luabind::call_function<luabind::object>(myLuaState, "MakeNewObjectFromLuaAndCPP", object, new GameObject())[ adopt(_2) ];
	lua_gc(myLuaState, LUA_GCCOLLECT, 0);
	Log("<GC>");

	const luabind::object& f = o["OnSpawn"];
	if (f)
		luabind::call_function<bool>(f);

//	ss<<"Result: " <<  << endl;
	string str = ss.str();

	//     (    )
	const luabind::object& objects = luabind::globals(myLuaState);
	for (luabind::iterator itr(objects); itr!=luabind::iterator(); itr++)
	{
		const luabind::object& obj = (*itr);
		luabind::object key = itr.key(); //  
		std::string name = luabind::object_cast<std::string>(key);
/*		string s = name + ": ";
		switch(type( obj ))
		{
		case LUA_TNIL: s += "LUA_TNIL"; break;
		case LUA_TBOOLEAN: s += "LUA_TBOOLEAN"; break;
		case LUA_TLIGHTUSERDATA: s += "LUA_TLIGHTUSERDATA"; break;
		case LUA_TNUMBER: s += "LUA_TNUMBER"; break;
		case LUA_TSTRING: s += "LUA_TSTRING"; break;
		case LUA_TTABLE: s += "LUA_TTABLE"; break;
		case LUA_TFUNCTION: s += "LUA_TFUNCTION"; break;
		case LUA_TUSERDATA: s += "LUA_TUSERDATA"; break;
		case LUA_TTHREAD: s += "LUA_TTHREAD"; break;
		}
		Log(s);*/
		if ( type( obj ) == LUA_TTABLE ) //  
		{
			//     
			for (luabind::iterator itr2(obj); itr2!=luabind::iterator(); itr2++)
			{
				//  ,     ?
				if (luabind::object_cast<std::string>(itr2.key()).compare("spriteFile")==0)
				{
					vecEnemyTypes.push_back(name);
				}
			}
		}
	}
	return true;
}

